/*
 Display the temperature and atmospheric pressure on a 1.8" TFT LCD screen.
 
 The display  coonections and graphics  libraries needed are detailed here:
 http://www.instructables.com/id/Arduino-TFT-display-and-font-library/
 
 The sensor is a BMP180 or BMP085.
 
 This examples uses the hardware SPI only. Non-hardware SPI
 is just too slow (~8 times slower!)
 
 Alan Senior 13/1/2015
 
 Colours:
 
 code	color
 0x0000	Black
 0xFFFF	White
 0xBDF7	Light Grey
 0x7BEF	Dark Grey
 0xF800	Red
 0xFFE0	Yellow
 0xFBE0	Orange
 0x79E0	Brown
 0x7E0	Green
 0x7FF	Cyan
 0x1F	Blue
 0xF81F	Pink
 
 */

#include<stdlib.h>

// Define the TFT pins and libraries
#define clk 13  // Don't change
#define din 11  // Don't change
#define cs   9
#define dc   8
#define rst  7  // you can also connect this to the Arduino reset
#include <Adafruit_GFX_AS.h>    // Core graphics library
#include <Adafruit_QDTech_AS.h> // Hardware-specific library
#include <SPI.h>

Adafruit_QDTech_AS tft = Adafruit_QDTech_AS(cs, dc, rst);       // Create the tft object

// Pull in the BMP180 (same as BMP085) and Wire libraries
#include <Wire.h>
#include <Adafruit_BMP085.h>

Adafruit_BMP085 bmp; // Create the bmp object


#define INTERVAL 3000 // Update interval for the display 3000 = 3 seconds
unsigned long startTime = 0;
float t = -15.0;

void setup(void) {
  tft.init();
  tft.setRotation(1);
  bmp.begin();
  startTime = millis() - INTERVAL; // So it starts staright away
  
  // Top and bottom borders
  for (int i = 0; i<160; i++) tft.drawFastVLine(i, 0, tft.height(), rainbow((i*128/160)%128));
  tft.fillRect(0,4,160,119, QDTech_BLACK);
}

void loop() {
  if (millis()-startTime > INTERVAL) { // Update the display every INTERVAL milliseconds
    startTime = millis();
    tft.setTextColor(QDTech_WHITE, QDTech_BLACK);

    int xpos = 5;
    int ypos = 7;

    tft.drawString("Temperature",xpos,ypos,2);

    t = bmp.readTemperature();
    //t = (random(600)-200)/10.0; // Generate a random temperature in range -20.0 to +40.0 for testing
    int col = t*10;
    col = constrain(col, 0, 300);
    col = map(col, 0, 300, 0, 127);
    tft.setTextColor(rainbow(col), QDTech_BLACK);
    xpos = 125;
    ypos = 25;
    char buf[20];
    dtostrf(t,6,1,buf);
    tft.drawRightString(buf,xpos,ypos,6);

    xpos+=3;
    xpos+=tft.drawString("o",xpos,ypos-5,2);    
    tft.drawString("C",xpos,ypos,4);

    xpos = 5;
    ypos += 50;
    tft.setTextColor(QDTech_WHITE, QDTech_BLACK);
    tft.drawString("Pressure",xpos,ypos,2);
 
    long p = bmp.readPressure();
    //p = (random(10000) + 95000); // Generate a random pressure in range 950 to 1050 for testing
    col = constrain(p/100, 970, 1030); // Pressure range for the colour selection
    col = map(col, 970, 1030, 0, 127);
    tft.setTextColor(rainbow(col), QDTech_BLACK);
    dtostrf((float)p/100.0,8,1,buf);
    xpos=100;
    ypos+=18;
    tft.drawRightString(buf,xpos,ypos,4);
    tft.drawString(" mb",xpos,ypos,4);
  }
}

unsigned int rainbow(byte value)
{
  // Value is expected to be in range 0-127
  // The value is converted to a spectrum colour from 0 = blue through to red = blue

  byte red = 0; // Red is the top 5 bits of a 16 bit colour value
  byte green = 0;// Green is the middle 6 bits
  byte blue = 0; // Blue is the bottom 5 bits

  byte quadrant = value/32;

  if (quadrant == 0) {
    blue = 31;
    green = 2*(value%32);
    red = 0;
  }
  if (quadrant == 1) {
    blue = 31 - (value%32);
    green = 63;
    red = 0;
  }
  if (quadrant == 2) {
    blue = 0;
    green = 63;
    red = value%32;
  }
  if (quadrant == 3) {
    blue = 0;
    green = 63-2*(value%32);
    red = 31;
  }
  return (red<<11) + (green<<5) + blue;
}




